
; DSTRIP - DOS filter to remove leading and trailing blanks, blank
;          lines and comment lines from dBASE command files, and to
;          capitalize text not in quotation marks.
;
comseg  segment para public 'code'
        assume  cs:comseg,ds:comseg,es:comseg,ss:comseg
        org     100h
start   proc    far
        cld
        mov     bl,00000001b    ;the bits in the BL register will hold
                                ;the following information when set to 1:
                                ;0-bit  only tabs and blanks encountered
                                ;       in current line
                                ;1-bit  a '*' encountered as first
                                ;       character of line
                                ;4-bit  char inside 'single' quotes
                                ;5-bit  char inside "double" quotes
                                ;6-bit  last char was linefeed
                                ;7-bit  last char in *-line was a ';'
        xor     dx,dx           ;DX will count blanks/tabs after a char
load:   push    bx              ;save registers and
        push    dx
        mov     dx,offset data  ;read into memory
        mov     cx,64000        ;64K bytes
        xor     bx,bx           ;from standard input
        mov     ah,3fh          ;with DOS function call
        int     21h
        pop     dx              ;restore registers
        pop     bx
        or      ax,ax           ;AX has number of bytes read; test for 0
        jnz     load0
        jmp     done
load0:  mov     cx,ax           ;move count into CX
        mov     si,offset data  ;and position index registers for
        mov     di,offset data  ;a string move
scan0:  lodsb                   ;fetch a byte into AL
        cmp     al,26           ;end of file?
        jz      store           ;if so, store and get out
        test    bl,01000000b    ;LF on previous fetch?
        jz      next            ;if not, skip
        or      bl,1            ;set beginning of line bit
        and     bl,10111111b    ;and reset the LF bit
        test    bl,10000000b    ;a ';' at end of line?
        jnz     next0           ;if yes, skip
        and     bl,11111101b    ;otherwise reset '*'-bit
next0:  and     bl,01111111b    ;reset ';'-bit
next:   cmp     al,59           ;a ';'?
        jnz     next1           ;if not, skip
        or      bl,10000000b    ;set ';'-bit
        jmp     short next5     ;and do loop routine
next1:  cmp     al,32           ;blank?
        jz      count           ;if yes, check whether to count it
        cmp     al,09           ;tab?
        jnz     next2           ;if not, skip count
count:  test    bl,11b          ;are we at beginning of line or in a '*'-line?
        jnz     loop            ;if yes, loop and omit
        inc     dx              ;otherwise count the blank/tab
        jmp     short store     ;and store it
next2:  cmp     al,13           ;CR?
        jz      out             ;if yes go to end of line routine
next3:  cmp     al,10           ;LF?
        jnz     next4           ;if not, go on checking
        or      bl,01000000b    ;set end of line bit
        jmp     short out       ;and go to end of line routine
next4:  test    bl,1            ;have we seen any nonblank/tab char before?
        jz      next6           ;if yes, ignore the next step
        cmp     al,42           ;a '*'?
        jnz     next5           ;if not, skip
        or      bl,10b          ;otherwise set the '*'-bit
next5:  and     bl,11111110b    ;and in either case we have nonblank/tab char
next6:  xor     dx,dx           ;set blank/tab count to 0
out:    test    bl,11b          ;are we at beginning of line or in a '*'-line?
        jnz     loop            ;if so, ignore (includes CR/LF of blank lines)
        sub     di,dx           ;DX is non-zero only if we encountered a CR
        xor     dx,dx           ;reset DX to 0
        and     bl,01111111b    ;reset ';'-bit
caps0:  jmp     short caps      ;capitalize
store:  stosb                   ;store char
loop:   cmp     al,26           ;test for end of file
        loopnz  scan0           ;and loop for next fetch
        push    ax              ;save registers
        push    bx
        push    dx
        mov     dx,offset data  ;compute the numbers of bytes stored
        mov     cx,di
        sub     cx,dx           ;in CX, and
        mov     bx,01           ;write to standard output
        mov     ah,40h          ;with DOS function call
        int     21h
        pop     dx
        pop     bx
        pop     ax
        cmp     al,26           ;end of file?
        jz      done
        jmp     load            ;if not, read another 64K bytes
done:   int     20h             ;end program
caps:   cmp     al,39           ;single quote?
        jnz     quot            ;if not, check double quote
        test    bl,100000b      ;is double quote bit set?
        jnz     store           ;if yes, ignore
        xor     bl,010000b      ;(re-)set single quote bit
        jmp     short store
quot:   cmp     al,34           ;double quote?
        jnz     caps01          ;if not, skip
        xor     bl,00100000b    ;(re-)set double-quote bit
        jmp     short store
caps01: test    bl,00110000b    ;any quote-bits set?
        jnz     store           ;if so, don't capitalize
caps1:  cmp     al,97           ;below 'a'?
        jb      store           ;if so, leave it alone
caps2:  cmp     al,122          ;above 'z'?
        ja      store           ;if so, don't change
caps3:  and     al,5fh          ;otherwise capitalize
out2:   jmp     short store
start   endp
data:
comseg  ends
        end     start
